@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2010 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -119,6 +119,17 @@ check_inactivity_timeout (struct context *c)
 }
 
 #if P2MP
+
+static inline void
+check_server_poll_timeout (struct context *c)
+{
+  void check_server_poll_timeout_dowork (struct context *c);
+
+  if (c->options.server_poll_timeout
+      && event_timeout_trigger (&c->c2.server_poll_interval, &c->c2.timeval, ETT_DEFAULT))
+    check_server_poll_timeout_dowork (c);
+}
+
 /*
  * Scheduled exit?
  */
@@ -224,10 +235,17 @@ get_link_socket_info (struct context *c)
 }
 
 static inline void
-register_activity (struct context *c)
+register_activity (struct context *c, const int size)
 {
   if (c->options.inactivity_timeout)
-    event_timeout_reset (&c->c2.inactivity_interval);
+    {
+      c->c2.inactivity_bytes += size;
+      if (c->c2.inactivity_bytes >= c->options.inactivity_minimum_bytes)
+	{
+	  c->c2.inactivity_bytes = 0;
+	  event_timeout_reset (&c->c2.inactivity_interval);
+	}
+    }
 }
 
 /*
