@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2010 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -34,6 +34,9 @@
 
 #include "buffer.h"
 
+struct openvpn_sockaddr;
+struct link_socket_actual;
+
 struct socks_proxy_info {
   bool defined;
   bool retry;
@@ -44,10 +47,12 @@ struct socks_proxy_info {
 
 void socks_adjust_frame_parameters (struct frame *frame, int proto);
 
-struct socks_proxy_info *new_socks_proxy (const char *server,
+struct socks_proxy_info *socks_proxy_new (const char *server,
 					  int port,
 					  bool retry,
-					  struct gc_arena *gc);
+					  struct auto_proxy_info *auto_proxy_info);
+
+void socks_proxy_close (struct socks_proxy_info *sp);
 
 void establish_socks_proxy_passthru (struct socks_proxy_info *p,
 				     socket_descriptor_t sd, /* already open to proxy */
@@ -58,14 +63,14 @@ void establish_socks_proxy_passthru (struct socks_proxy_info *p,
 void establish_socks_proxy_udpassoc (struct socks_proxy_info *p,
 				     socket_descriptor_t ctrl_sd, /* already open to proxy */
 				     socket_descriptor_t udp_sd,
-				     struct sockaddr_in *relay_addr,
+				     struct openvpn_sockaddr *relay_addr,
 				     volatile int *signal_received);
 
 void socks_process_incoming_udp (struct buffer *buf,
-				struct sockaddr_in *from);
+				struct link_socket_actual *from);
 
 int socks_process_outgoing_udp (struct buffer *buf,
-				struct sockaddr_in *to);
+				const struct link_socket_actual *to);
 
 #endif
 #endif
